//    ******************************************
//    *                                        *
//    *  package:  LabPro   v1.1               *
//    *  author:   Jochen Viehoff              *
//    *                                        *
//    *  JAVA source code for Vernier LabPro   *
//    *  data collection, single sensor mode   *
//    *                                        *
//    *  communicationAPI required             *
//    *  real-time data collection             *
//    *  binary data (serial port)             *
//    *                                        *
//    ******************************************

// -----------------------------------------------

//    ******************************************
//    * Sensor: Barometer                      *
//    *                                        *
//    * class:   Barometer                     *
//    *                                        *
//    * methods:  (double) getPressure()       *
//    *                                        *
//    * parameter: sampleRate                  *
//    *            threadDelay                 *
//    *                                        *
//    ******************************************

import java.io.*;
import java.util.*;
import javax.comm.*;                          // commAPI for serial communication

public class Barometer implements Runnable, SerialPortEventListener {
    
    static Enumeration        portList;       // list for connected ports
    static CommPortIdentifier portId;
    
    static float sampleRate  = 0.02f;         // sample rate for real-time data collection
    
    final int     threadDelay = 10000;        // thread delay
    int           count;
    static int    raw;                       // raw binary data
    static double p;                         // pressure
    
    static double k0 =  819.52;                // gauge parameters
    static double k1 =  0.0005951;       
    
    static String message0;                   // commands for LabPro setup
    static String message1;
    static String messageStart;
    static String messageReset;
    
    static SerialPort   serialPort;
    static OutputStream outputStream;
    static InputStream  inputStream;
    
    Thread readData;
    
    //    ******************************************
    //      constructor
    //    ******************************************
    
    public Barometer() {
        
        portList = CommPortIdentifier.getPortIdentifiers();
        
        // locking for serial ports
        
        while (portList.hasMoreElements()) {
            portId = (CommPortIdentifier) portList.nextElement();
            if (portId.getPortType() == CommPortIdentifier.PORT_SERIAL) {
                System.out.println("Found: SerialPort");
                if (portId.getName().equals("COM1")) {
                    System.out.println("Found COM1");
                    
                    // open port COM1
                    try {
                        serialPort = (SerialPort) portId.open("LabPro", 2000);
                        System.out.println("open port: done");
                    } catch (PortInUseException e) {System.out.println(e);}
                    
                    // get the streams from/to port COM1
                    try {
                        inputStream = serialPort.getInputStream();
                        outputStream= serialPort.getOutputStream();
                    } catch (IOException e) {System.out.println(e);}
                    
                    try {
                        serialPort.addEventListener(this);
                    } catch (TooManyListenersException e) {System.out.println(e);}
                    
                    // start listening to the port
                    serialPort.notifyOnDataAvailable(true);
                    
                    // set port parameters
                    try {
                        serialPort.setSerialPortParams(38400,
                        SerialPort.DATABITS_8,
                        SerialPort.STOPBITS_1,
                        SerialPort.PARITY_NONE);
                    } catch (UnsupportedCommOperationException e) {System.out.println(e);}
                    
                    // start thread for checking data rate
                    readData = new Thread(this);
                    readData.start();
                }
            }
        }
    }
    
    //    ******************************************
    //      public methods
    //    ******************************************
    
    public static double getPressure() {
        
        // send back pressure
        p = (double)((k1*raw)+k0);
        return p;
    }

    public void run() {
        
        // reset LabPro
        messageReset = "s{0}\n";
        System.out.println("Reset LabPro");
        
        // build commands for channel set up
        message0 = "s{1,1,1,0,0,1}\n";
        message1 = "s{4,0,-1}\n";            //request binary data !!!!
        
        // start LabPro: real time sampling
        messageStart = "s{3,"+sampleRate+",-1}\n";
        System.out.println("start real time sampling: t="+sampleRate);
        
        // send commands to LabPro
        try {
            outputStream.write(messageReset.getBytes());
            outputStream.write(message0.getBytes());
            outputStream.write(message1.getBytes());
            outputStream.write(messageStart.getBytes());
        } catch (IOException e) {}
        
        //  check sampling rate
        while (readData != null) {
            try {
                Thread.sleep(threadDelay);
            } catch (InterruptedException e) {}
            
            System.out.println(count);
            count=0;
        }
        
        //  close port if thread has died
        serialPort.close();
    }
    
    //    ******************************************
    //      SerialEvent handling
    //    ******************************************
    
    public void serialEvent(SerialPortEvent e) {
        
        switch (e.getEventType()) {
            
            case SerialPortEvent.OUTPUT_BUFFER_EMPTY:
                break;
            case SerialPortEvent.DATA_AVAILABLE:
                
                final int BUFFSIZE=12;
                int lenserial=0;
                byte bufserial [] = new byte[BUFFSIZE];
                
                // Read binary data until -1 is returned
                try {
                    while (inputStream.available() > 0) {
                        lenserial = inputStream.read(bufserial);
                        //  System.out.println(lenserial);
                        if (lenserial==4){
                            raw = ((0xff & bufserial[0])<<8) | ((0xff & bufserial[1]));
                            count++;
                            // System.out.println(rawX+" "+rawY+" "+rawZ);
                        }
                    }
                }
                catch (IOException ex) {
                    System.err.println(ex);
                    return;
                }
                break;
                
                // If break event append BREAK RECEIVED message.
                
            case SerialPortEvent.BI:
                System.out.println("\n--- BREAK RECEIVED ---\n");
        }
    }
}
