//    ******************************************
//    *                                        *
//    *  package:  LabPro   v1.1               *
//    *  author:   Jochen Viehoff              *
//    *                                        *
//    *  JAVA source code for Vernier LabPro   *
//    *  data collection, single sensor mode   *
//    *                                        *
//    *  communicationAPI required             *
//    *  real-time data collection             *
//    *  binary data (serial port)             *
//    *                                        *
//    ******************************************

// -----------------------------------------------

//    ******************************************
//    * Sensor:   Digital Output Controller    *
//    *                                        *
//    * class:    DigitalOut                   *
//    *                                        *
//    * methods:  (void)   setOutput(int)      *
//    *                                        *
//    *     int 0:    0 0 0 0 0 0 0 0          *
//    *     int 1:    0 0 0 0 0 0 0 1          *
//    *     int 2:    0 0 0 0 0 0 1 0          *
//    *     int 3:    0 0 0 0 0 0 1 1          *
//    *     int 4:    0 0 0 0 0 1 0 0          *
//    *     int 5:    0 0 0 0 0 1 0 1          *
//    *     int 6:    0 0 0 0 0 1 1 0          *
//    *     int 7:    0 0 0 0 0 1 1 1          *
//    *     int 8:    0 0 0 0 1 0 0 0          *
//    *                                        *
//    *      etc.                              *
//    *                                        *
//    * parameter: sampleRate                  *
//    *            threadDelay                 *
//    *                                        *
//    ******************************************

import java.io.*;
import java.util.*;
import javax.comm.*;                          // commAPI for serial communication

public class DigitalOut implements SerialPortEventListener {
    
    static Enumeration        portList;       // list for connected ports
    static CommPortIdentifier portId;
    
    static String message;                   // commands for LabPro setup
    static String messageReset;
    
    static SerialPort   serialPort;
    static OutputStream outputStream;
    static InputStream  inputStream;
    
    
    //    ******************************************
    //      constructor
    //    ******************************************
    
    public DigitalOut() {
        
        portList = CommPortIdentifier.getPortIdentifiers();
        
        // locking for serial ports
        
        while (portList.hasMoreElements()) {
            portId = (CommPortIdentifier) portList.nextElement();
            if (portId.getPortType() == CommPortIdentifier.PORT_SERIAL) {
                System.out.println("Found: SerialPort");
                if (portId.getName().equals("COM1")) {
                    System.out.println("Found COM1");
                    
                    // open port COM1
                    try {
                        serialPort = (SerialPort) portId.open("LabPro", 2000);
                        System.out.println("open port: done");
                    } catch (PortInUseException e) {System.out.println(e);}
                    
                    // get the streams from/to port COM1
                    try {
                        inputStream = serialPort.getInputStream();
                        outputStream= serialPort.getOutputStream();
                    } catch (IOException e) {System.out.println(e);}
                    
                    try {
                        serialPort.addEventListener(this);
                    } catch (TooManyListenersException e) {System.out.println(e);}
                    
                    // start listening to the port
                    serialPort.notifyOnDataAvailable(true);
                    
                    // set port parameters
                    try {
                        serialPort.setSerialPortParams(38400,
                        SerialPort.DATABITS_8,
                        SerialPort.STOPBITS_1,
                        SerialPort.PARITY_NONE);
                    } catch (UnsupportedCommOperationException e) {System.out.println(e);}
                    
                }
            }
        }
        
        // reset LabPro
        messageReset = "s{0}\n";
        try {outputStream.write(messageReset.getBytes()); }
        catch (IOException e) {}
        System.out.println("Reset LabPro");
    }
    
    //    ******************************************
    //      public methods
    //    ******************************************
    
    public static void setLevel(int output) {
        
        // send back level of table
        message = "s{2001,"+output+"}\n";
        try {outputStream.write(message.getBytes()); }
        catch (IOException e) {}
    }
    //    ******************************************
    //      SerialEvent handling
    //    ******************************************
    
    public void serialEvent(SerialPortEvent e) {
        
        switch (e.getEventType()) {
            
            case SerialPortEvent.OUTPUT_BUFFER_EMPTY:
                break;
            case SerialPortEvent.DATA_AVAILABLE:
                
                final int BUFFSIZE=10;
                int lenserial=0;
                byte bufserial [] = new byte[BUFFSIZE];
                
                break;
                
                // If break event append BREAK RECEIVED message.
                
            case SerialPortEvent.BI:
                System.out.println("\n--- BREAK RECEIVED ---\n");
        }
    }
    
}
