//    ******************************************
//    *                                        *
//    *  package:  LabPro   v1.1               *
//    *  author:   Jochen Viehoff              *
//    *                                        *
//    *  JAVA source code for Vernier LabPro   *
//    *  data collection, single sensor mode   *
//    *                                        *
//    *  communicationAPI required             *
//    *  real-time data collection             *
//    *  binary data (serial port)             *
//    *                                        *
//    ******************************************

// -----------------------------------------------

//    ******************************************
//    * Sensor: Motion Detection               *
//    *                                        *
//    * class:    MotionDetection              *
//    *                                        *
//    * methods:  (double) getDistance()       *
//    *           (double) getVelocity()       *
//    *           (double) getAcceleration()   *
//    *           (double) getTime()           *
//    *                                        *
//    * parameter: sampleRate                  *
//    *            threadDelay                 *
//    *                                        *
//    ******************************************

import java.io.*;
import java.util.*;
import javax.comm.*;                   // commAPI for serial communication

public class MotionDetection implements Runnable, SerialPortEventListener {
    
    static Enumeration        portList;    // list for connected ports
    static CommPortIdentifier portId;
    
    static final float sampleRate  = 0.1f;       // sample rate for real-time data collection
    final int          threadDelay = 10000;       // thread delay for data rate checking
    int count;
    
    static String messageString0 = "s{0}\n";
    static String messageString1 = "s{1,11,1,2}\n";
    static String messageString2 = "s{3,"+sampleRate+",-1}\n";
    
    static SerialPort   serialPort;
    static OutputStream outputStream;
    static InputStream  inputStream;
    
    static float f1,f2,f3,f4;                       // floats for d,v,a and t
    Thread    readData;
    
    //    ******************************************
    //      constructor
    //    ******************************************
    
    public MotionDetection() {
        
        portList = CommPortIdentifier.getPortIdentifiers();
        
        // locking for serial ports
        
        while (portList.hasMoreElements()) {
            portId = (CommPortIdentifier) portList.nextElement();
            if (portId.getPortType() == CommPortIdentifier.PORT_SERIAL) {
                System.out.println("Found: SerialPort");
                if (portId.getName().equals("COM1")) {
                    System.out.println("Found COM1");
                    
                    // open port COM1
                    try {
                        serialPort = (SerialPort) portId.open("LabPro", 2000);
                        System.out.println("open port: done");
                    } catch (PortInUseException e) {System.out.println(e);}
                    
                    // get the streams from/to port COM1
                    try {
                        inputStream = serialPort.getInputStream();
                        outputStream= serialPort.getOutputStream();
                    } catch (IOException e) {System.out.println(e);}
                    
                    try {
                        serialPort.addEventListener(this);
                    } catch (TooManyListenersException e) {System.out.println(e);}
                    
                    // start listening to the port
                    serialPort.notifyOnDataAvailable(true);
                    
                    // set port parameters
                    try {
                        serialPort.setSerialPortParams(38400,
                        SerialPort.DATABITS_8,
                        SerialPort.STOPBITS_1,
                        SerialPort.PARITY_NONE);
                    } catch (UnsupportedCommOperationException e) {System.out.println(e);}
                    
                    // start thread for data collection
                    System.out.println("start real time data collection");
                    readData = new Thread(this);
                    readData.start();
                }
            }
        }
    }
    
    //    ******************************************
    //      public methods
    //    ******************************************
    
    public static double getDistance() {
        return (double)f1;
    }
    
    public static double getVelocity() {
        return (double)f2;
    }
    
    public static double getAcceleration() {
        return (double)f3;
    }
    
    public static double getTime() {
        return (double)f4;
    }
    
    public void run() {
        
        //  start data collection with LabPro
        System.out.println("start sending data");
        try {
            outputStream.write(messageString0.getBytes());
            outputStream.write(messageString1.getBytes());
            outputStream.write(messageString2.getBytes());
        } catch (IOException e) {}
        
        //  loop for real-time data collection
        while (readData != null) {
            try {
                Thread.sleep(threadDelay);
            } catch (InterruptedException e) {}
            System.out.println(count);
            count=0;
        }
        
        //  close port if thread has died
        serialPort.close();
    }
    
    //    ******************************************
    //      SerialEvent handling
    //    ******************************************
    
    public void serialEvent(SerialPortEvent e) {
        
        // Create a StringBuffer and int to receive input data.
        
        StringBuffer inputBuffer = new StringBuffer();
        int newData=0;
        int i=0;
        String s1="";
        
        // Determine type of event.
        switch (e.getEventType()) {
            
            // Read data until -1 is returned. If \r is received substitute
            // \n for correct newline handling.
            case SerialPortEvent.DATA_AVAILABLE:
                
                while (newData != -1) {
                    try {
                        newData = inputStream.read();
                        i++;
                        if (newData == -1) {
                            System.out.println("-1 received !!");
                            break;
                        }
                        if ('\r' == (char)newData) {
                            
                            // get float data from string
                            try {
                                f1 = Float.valueOf(s1.substring(4,16).trim()).floatValue();
                                f2 = Float.valueOf(s1.substring(18,30).trim()).floatValue();
                                f3 = Float.valueOf(s1.substring(32,44).trim()).floatValue();
                                f4 = Float.valueOf(s1.substring(46,58).trim()).floatValue();
                            }
                            catch(NumberFormatException nfe){}
                            count++;
                            s1="";
                            i++;
                        }
                        else {
                            s1=s1+(char)newData;
                        }
                    }
                    catch (IOException ex) {
                        System.err.println(ex);
                        return;
                    }
                }
                
                break;
                
                // If break event append BREAK RECEIVED message.
            case SerialPortEvent.BI:
                System.out.println("\n--- BREAK RECEIVED ---\n");
        }
        
    }
}
