//    ******************************************
//    *                                        *
//    *  package:  LabPro   v1.1               *
//    *  author:   Jochen Viehoff              *
//    *                                        *
//    *  JAVA source code for Vernier LabPro   *
//    *  data collection, single sensor mode   *
//    *                                        *
//    *  communicationAPI required             *
//    *  real-time data collection             *
//    *  binary data (serial port)             *
//    *                                        *
//    ******************************************


// -----------------------------------------------

//    ******************************************
//    * Sensor: Temperature                    *
//    *                                        *
//    * class:    Temperature                  *
//    *                                        *
//    * methods:  (double) getTemperature()    *
//    *                                        *
//    * parameter: sampleRate                  *
//    *            threadDelay                 *
//    *                                        *
//    ******************************************

import java.io.*;
import java.util.*;
import javax.comm.*;                          // commAPI for serial communication

public class Temperature implements Runnable, SerialPortEventListener {
    
    static Enumeration        portList;       // list for connected ports
    static CommPortIdentifier portId;
    
    static final float sampleRate  = 0.01f;   // sample rate for real-time data collection
    
    final int          threadDelay = 10000;   // thread delay for data collection
    
    static String messageString0;             // commands for LabPro init
    static String messageStart;
    static String messageReset;
    
    static SerialPort   serialPort;
    static OutputStream outputStream;
    static InputStream  inputStream;
    
    static float f1;                          // float for temperature
    int count;
    Thread    readData;                       // thread for checking data rate
    
    //    ******************************************
    //      constructor
    //    ******************************************
    
    public Temperature() {
        
        portList = CommPortIdentifier.getPortIdentifiers();
        
        // locking for serial ports
        while (portList.hasMoreElements()) {
            portId = (CommPortIdentifier) portList.nextElement();
            if (portId.getPortType() == CommPortIdentifier.PORT_SERIAL) {
                System.out.println("Found: SerialPort");
                if (portId.getName().equals("COM1")) {
                    System.out.println("Found COM1");
                    
                    // open port COM1
                    try {
                        serialPort = (SerialPort) portId.open("LabPro", 2000);
                        System.out.println("open port: done");
                    } catch (PortInUseException e) {System.out.println(e);}
                    
                    // get the streams from/to port COM1
                    try {
                        inputStream = serialPort.getInputStream();
                        outputStream= serialPort.getOutputStream();
                    } catch (IOException e) {System.out.println(e);}
                    
                    try {
                        serialPort.addEventListener(this);
                    } catch (TooManyListenersException e) {System.out.println(e);}
                    
                    // start listening to the port
                    serialPort.notifyOnDataAvailable(true);
                    
                    // set port parameters
                    try {
                        serialPort.setSerialPortParams(38400,
                        SerialPort.DATABITS_8,
                        SerialPort.STOPBITS_1,
                        SerialPort.PARITY_NONE);
                    } catch (UnsupportedCommOperationException e) {System.out.println(e);}
                    
                    // start thread for checking data rate
                    readData = new Thread(this);
                    readData.start();
                }
            }
        }
    }
    
    //    ******************************************
    //      public methods
    //    ******************************************
    
    public static double getTemperature() {
        
        return (double)f1;      // send back temperature in celsius
    }
    
    public void run() {
        
        // reset LabPro
        messageReset = "s{0}\n";
        System.out.println("Reset LabPro");
        
        // build commands for channel set up (no binary data !!!)
        messageString0 = "s{1,1,1,0,0,0}\n";
        
        // start LabPro: real time sampling
        messageStart = "s{3,"+sampleRate+",-1}\n";
        System.out.println("start real time sampling: t="+sampleRate);
        
        // send commands to LabPro
        try {
            outputStream.write(messageReset.getBytes());
            outputStream.write(messageString0.getBytes());
            outputStream.write(messageStart.getBytes());
        } catch (IOException e) {}
        
        //  loop for real-time data collection
        while (readData != null) {
            try {
                Thread.sleep(threadDelay);
            } catch (InterruptedException e) {}
            System.out.println(count);
            count = 0;
        }
        
        //  close port if thread has died
        serialPort.close();
    }
    
    //    ******************************************
    //      SerialEvent handling
    //    ******************************************
    
    public void serialEvent(SerialPortEvent e) {
        
        // Create a StringBuffer and int to receive input data.
        StringBuffer inputBuffer = new StringBuffer();
        int newData=0;
        int i=0;
        String s1="";
        
        // Determine type of event
        switch (e.getEventType()) {
            
            // Read data until -1 is returned
            case SerialPortEvent.DATA_AVAILABLE:
                while (newData != -1) {
                    try {
                        newData = inputStream.read();
                        i++;
                        if (newData == -1) {
                            System.out.println("-1 received !!");
                            break;
                        }
                        if ('\r' == (char)newData) {
                            
                            // get float data from string
                            try {
                                f1 = Float.valueOf(s1.substring(4,16).trim()).floatValue();
                            }
                            catch(NumberFormatException nfe){}
                            count++;
                            s1="";
                            i++;
                        }
                        else {
                            s1=s1+(char)newData;
                        }
                    }
                    catch (IOException ex) {
                        System.err.println(ex);
                        return;
                    }
                }
                
                break;
                
                // If break event append BREAK RECEIVED message.
                
            case SerialPortEvent.BI:
                System.out.println("\n--- BREAK RECEIVED ---\n");
        }
        
    }
}
